from django_filters import rest_framework as filters
from rest_framework import serializers, viewsets
from views import MunityViewSet

from .models import Record

class RecordSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "id",
            "previous_value",
            "next_value",
            "created",
            "user",
            "product",
        ]
        model = Record

class RecordsFilter(filters.FilterSet):
    class Meta:
        fields = {
            "created": ["gt", "gte", "lt", "lte"],
            "user": ["exact"],
            "product": ["exact"],
        }
        model = Record


class RecordsViewSet(MunityViewSet):
    serializer_class = RecordSerializer
    filterset_class = RecordsFilter




from django.db.models.query_utils import Q
from munity.authorization.models import Permission
from munity.users.models import UserRoleWorkspace
from rest_framework import permissions

from ..utils import get_request_info

class HasEndpointPermission(permissions.BasePermission):

    def has_permission(self, request, view):
        if request.user.is_superuser:
            return True

        (workspace_slug, ressource, action) = get_request_info(request)
        # searching for needed permission
        needed_permission = Permission.objects.filter(
            action=action,
            ressource__app_label=ressource
        )

        # All user can access roles
        if action == 'list' and ressource == 'roles':
            return True

        # checking if user as at least one role that fit needed permission on wanted workspace or on all workspace
        has_role_permission = UserRoleWorkspace.objects.filter(
            Q(user=request.user) &
            Q(Q(workspace=workspace_slug) | Q(workspace=None)) &
            Q(role__permissions__in=needed_permission)
        )

        return has_role_permission


from functools import partial
from django_filters import rest_framework as filters
from ..files.views import FileSerializer
from rest_framework import serializers, viewsets
from django.db.models.query_utils import Q
from .models import User, UserRoleWorkspace
from ..models import MunityGroupableModel
from ..workspaces.models import Workspace
from django.contrib.contenttypes.models import ContentType

from ..utils import UUIDEncoder
from ..records.models import Record
from deepdiff import DeepDiff
import json

from ..views import MunityViewSet


class UserRoleWorkspaceSerializer(serializers.ModelSerializer):
    class Meta:
        fields = ('workspace', 'role')
        model = UserRoleWorkspace

class UserSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "id",
            "username",
            "email",
            "first_name",
            "last_name",
            "created",
            "avatar",
            "is_superuser",
            "modified",
            "generic_groups",
            "user_role_workspaces",
            "password",
        ]
        model = User
        extra_kwargs = {
            'password': {
                'write_only': True
            }
        }

    password=serializers.CharField(required=False)
    user_role_workspaces = UserRoleWorkspaceSerializer(many=True)

    # We show full avatar in formation on get, on update we only use doc id
    def to_representation(self, instance):
        response = super().to_representation(instance)
        response['avatar'] = FileSerializer(instance.avatar).data if instance.avatar else None
        return response

    def update(self, instance, validated_data):
        if 'user_role_workspaces' in validated_data:
            user_role_workspaces_data = validated_data.pop('user_role_workspaces')

            user_workspaces = []
            for user_role_workspace_data in user_role_workspaces_data:
                user_workspaces.append(user_role_workspace_data.get("workspace"))
                UserRoleWorkspace.objects.update_or_create(
                    user=instance, **user_role_workspace_data
                )
            # remove removed rights
            UserRoleWorkspace.objects.filter(user=instance).exclude(workspace__in=user_workspaces).delete()

        user = super(self.__class__, self).update(instance, validated_data)
        if 'password' in validated_data:
            user.set_password(validated_data['password'])
            user.save()
        return user

    def create(self, validated_data):
        user_role_workspaces_data = validated_data.pop('user_role_workspaces')
        user = User.objects.create(**validated_data)
        for user_role_workspace_data in user_role_workspaces_data:
            UserRoleWorkspace.objects.create(
                user=user, **user_role_workspace_data
            )
        if 'password' in validated_data:
            user.set_password(validated_data['password'])
            user.save()
        return user


class UsersFilter(filters.FilterSet):
    class Meta:
        fields = {
            "id": ["exact", "in"],
            "first_name": ["exact", "in", "contains"],
            "generic_groups": ["in"],
            "last_name": ["exact", "in", "contains"],
            "email": ["exact", "in", "contains"],
            "username": ["exact", "in", "contains"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = User

class UsersViewSet(MunityViewSet, MunityGroupableModel):
    serializer_class = UserSerializer
    filterset_class = UsersFilter

    def get_queryset(self):
        model = self.serializer_class.Meta.model
        if "workspace_pk" in self.kwargs:
            return model.objects.filter(Q(id=self.request.user.id) | Q(user_role_workspaces__workspace=self.kwargs["workspace_pk"]))
        return model.objects.filter(is_superuser=True)




from django_filters import rest_framework as filters
from rest_framework import serializers, viewsets

from .models import Workspace


class WorkspaceSerializer(serializers.ModelSerializer):
    class Meta:
        fields = [
            "slug",
            "db_connection",
            "created",
            "modified",
        ]
        model = Workspace


class WorkspacesFilter(filters.FilterSet):
    class Meta:
        fields = {
            "slug": ["exact", "in"],
            "created": ["gt", "gte", "lt", "lte"],
            "modified": ["gt", "gte", "lt", "lte"],
        }
        model = Workspace


class WorkspacesViewSet(viewsets.ModelViewSet):
    queryset = Workspace.objects.all().order_by('slug')
    serializer_class = WorkspaceSerializer
    filterset_class = WorkspacesFilter




from django.contrib import admin
from .models import Settings

@admin.register(Settings)
class SettingsAdmin(admin.ModelAdmin):
    list_display = ("key", "value", "workspace")
    list_editable = ("value",)
    list_filter = ("key", "workspace")


from django.contrib.contenttypes.models import ContentType
from rest_framework import viewsets
from deepdiff import DeepDiff
from django.db.models.query_utils import Q

from .workspaces.models import Workspace
from .records.models import Record

from .utils import UUIDEncoder

import json

# Create your views here.
class MunityViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        model = self.serializer_class.Meta.model
        if "workspace_pk" in self.kwargs:
            return model.objects.filter(Q(workspace=None) | Q(workspace=self.kwargs["workspace_pk"]))
        return model.objects.all()

    def destroy(self, request, workspace_pk=None, pk=None):
        deleted_model = self.serializer_class.Meta.model.objects.get(pk=pk)
        ctype = ContentType.objects.get_for_model(deleted_model)
        response = super().destroy(request, workspace_pk, pk=pk)
        record = Record.objects.create(
            previous_value = None,
            diff_value = None,
            action = 'delete',
            workspace = Workspace.objects.get(slug=workspace_pk),
            user = request.user,
            product_object_id = pk,
            product_content_type = ctype
        )
        record.save()
        return response

    def create(self, request, workspace_pk=None):
        response = super().create(request, workspace_pk)
        model_id = dict(response.data).get('id')
        new_model = self.serializer_class.Meta.model.objects.get(pk=model_id)
        ctype = ContentType.objects.get_for_model(new_model)
        record = Record.objects.create(
            previous_value = None,
            diff_value = None,
            action = 'create',
            workspace = Workspace.objects.get(slug=workspace_pk),
            user = request.user,
            product_object_id = model_id,
            product_content_type = ctype
        )
        record.save()
        return response

    def update(self, request, workspace_pk=None, pk=None, partial=False):
        old_model = self.serializer_class.Meta.model.objects.get(pk=pk)
        response = super().update(request, workspace_pk, pk)
        new_model = self.serializer_class.Meta.model.objects.get(pk=pk)
        diff = DeepDiff(
            old_model,
            new_model,
            exclude_paths=["root.modified"]
        )
        ctype = ContentType.objects.get_for_model(old_model)
        if diff:
            record = Record.objects.create(
                previous_value = json.dumps(self.serializer_class(old_model).data, cls=UUIDEncoder),
                diff_value = diff,
                action = 'update',
                workspace = Workspace.objects.get(slug=workspace_pk),
                user = request.user,
                product_object_id = pk,
                product_content_type = ctype
            )
            record.save()
        return response

